package org.antlr.analysis;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.antlr.analysis.DFA;
import org.antlr.analysis.DFAState;
import org.antlr.analysis.NFAState;
import org.antlr.analysis.Transition;
import org.antlr.misc.IntSet;
import org.antlr.runtime.CommonToken;
import org.antlr.runtime.Token;
import org.antlr.tool.Grammar;

public class MachineProbe {
  DFA dfa;
  
  public MachineProbe(DFA dfa) {
    this.dfa = dfa;
  }
  
  List<DFAState> getAnyDFAPathToTarget(DFAState targetState) {
    Set<DFAState> visited = new HashSet<DFAState>();
    return getAnyDFAPathToTarget(this.dfa.startState, targetState, visited);
  }
  
  public List<DFAState> getAnyDFAPathToTarget(DFAState startState, DFAState targetState, Set<DFAState> visited) {
    List<DFAState> dfaStates = new ArrayList<DFAState>();
    visited.add(startState);
    if (startState.equals(targetState)) {
      dfaStates.add(targetState);
      return dfaStates;
    } 
    for (int i = 0; i < startState.getNumberOfTransitions(); i++) {
      Transition e = startState.getTransition(i);
      if (!visited.contains(e.target)) {
        List<DFAState> path = getAnyDFAPathToTarget((DFAState)e.target, targetState, visited);
        if (path != null) {
          dfaStates.add(startState);
          dfaStates.addAll(path);
          return dfaStates;
        } 
      } 
    } 
    return null;
  }
  
  public List<IntSet> getEdgeLabels(DFAState targetState) {
    List<DFAState> dfaStates = getAnyDFAPathToTarget(targetState);
    List<IntSet> labels = new ArrayList<IntSet>();
    for (int i = 0; i < dfaStates.size() - 1; i++) {
      DFAState d = dfaStates.get(i);
      DFAState nextState = dfaStates.get(i + 1);
      for (int j = 0; j < d.getNumberOfTransitions(); j++) {
        Transition e = d.getTransition(j);
        if (e.target.stateNumber == nextState.stateNumber)
          labels.add(e.label.getSet()); 
      } 
    } 
    return labels;
  }
  
  public String getInputSequenceDisplay(Grammar g, List<IntSet> labels) {
    List<String> tokens = new ArrayList<String>();
    for (IntSet label : labels)
      tokens.add(label.toString(g)); 
    return tokens.toString();
  }
  
  public List<Token> getGrammarLocationsForInputSequence(List<Set<NFAState>> nfaStates, List<IntSet> labels) {
    List<Token> tokens = new ArrayList<Token>();
    for (int i = 0; i < nfaStates.size() - 1; i++) {
      Set<NFAState> cur = nfaStates.get(i);
      Set<NFAState> next = nfaStates.get(i + 1);
      IntSet label = labels.get(i);
      label22: for (NFAState p : cur) {
        for (int j = 0; j < p.getNumberOfTransitions(); j++) {
          Transition t = p.transition(j);
          if (!t.isEpsilon() && !t.label.getSet().and(label).isNil() && next.contains(t.target))
            if (p.associatedASTNode != null) {
              Token oldtoken = p.associatedASTNode.token;
              CommonToken token = new CommonToken(oldtoken.getType(), oldtoken.getText());
              token.setLine(oldtoken.getLine());
              token.setCharPositionInLine(oldtoken.getCharPositionInLine());
              tokens.add(token);
              break label22;
            }  
        } 
      } 
    } 
    return tokens;
  }
}
